<?php declare(strict_types=1);

namespace BitBadger\PDODocument;

/**
 * Functions to patch (partially update) documents
 */
class Patch
{
    /**
     * Patch a document by its ID
     *
     * @param string $tableName The table in which the document should be patched
     * @param mixed $docId The ID of the document to be patched
     * @param array|object $patch The object with which the document should be patched (will be JSON-encoded)
     * @throws DocumentException If any is encountered (database mode must be set)
     */
    public static function byId(string $tableName, mixed $docId, array|object $patch): void
    {
        Custom::nonQuery(Query\Patch::byId($tableName),
            array_merge(Parameters::id($docId), Parameters::json(':data', $patch)));
    }

    /**
     * Patch documents using a comparison on JSON fields
     *
     * @param string $tableName The table in which documents should be patched
     * @param array|Field[] $fields The field comparison to match
     * @param array|object $patch The object with which the documents should be patched (will be JSON-encoded)
     * @param FieldMatch|null $match How to handle multiple conditions (optional; defaults to All)
     * @throws DocumentException If any is encountered
     */
    public static function byFields(string $tableName, array $fields, array|object $patch,
                                    ?FieldMatch $match = null): void
    {
        $namedFields = Parameters::nameFields($fields);
        Custom::nonQuery(Query\Patch::byFields($tableName, $namedFields, $match),
            Parameters::addFields($namedFields, Parameters::json(':data', $patch)));
    }
}
